/*
 * A simple pager exposing a single memory region.
 *
 * Copyright (C) 2021, 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/c/dataspace.h>

#include <mem/memory_incremental.h>

#include <string.h>

#include "dataspace_server.h"
#include "ipc.h"
#include "simple_pager.h"



SimplePager::SimplePager(Memory *memory)
{
    if (memory == NULL)
        _memory = new MemoryIncremental();
    else
        _memory = memory;

    _region = _memory->region();
}

void SimplePager::close()
{
    if (_region != NULL)
    {
        _memory->release(_region);
        _region = NULL;
    }
}



/* Data access methods. */

char *SimplePager::get_string(offset_t offset)
{
    char *buffer = _region->read(offset);
    offset_t size = _region->size();

    /* Confine the path to the limit of the buffer. */

    if ((buffer != NULL) && (strnlen(buffer, size) < size))
        return buffer;
    else
        return NULL;
}



/* Map a flexpage corresponding to the dataspace 'offset' involving a 'hot_spot'
   (flexpage offset). */

long SimplePager::map(offset_t offset, map_address_t hot_spot, map_flags_t flags, l4_snd_fpage_t *region)
{
    Flexpage flexpage(_region);

    flexpage.reset(offset);
    flexpage.upgrade(flags);

    /* Send the flexpage explicitly. */

    long err = ipc_prepare_flexpage(&flexpage, offset, _region->size(),
                                    hot_spot, flags, region);

    if (err)
        return err;

    err = complete_Dataspace_map(*region);

    if (err)
        return err;

    return IPC_MESSAGE_SENT;
}

long SimplePager::info(offset_t *size, map_flags_t *flags)
{
    *size = _region->size();
    *flags = L4_FPAGE_RW;

    return L4_EOK;
}

// vim: tabstop=4 expandtab shiftwidth=4
